#!/usr/bin/env python

"""
Tries to find the metadata of songs based on the file name
https://github.com/lakshaykalbhor/MusicRepair
"""

import difflib
import requests
import six
import spotipy

from os import rename

from mutagen import File
from mutagen.id3 import APIC, ID3, USLT, _util
from mutagen.mp3 import EasyMP3

if six.PY2:
    import BeautifulSoup
    from urllib2 import quote, urlopen

elif six.PY3:
    from bs4 import BeautifulSoup
    from urllib.parse import quote
    from urllib.request import urlopen

# Project specific inputs
import albumsearch
import improvename
import log
from utils import *

def matching_details(song_name, song_title, artist):
    """
    Provides a score out of 10 that determines the
    relevance of the search result
    """

    match_name = difflib.SequenceMatcher(None, song_name, song_title).ratio()
    match_title = difflib.SequenceMatcher(None, song_name, artist + song_title).ratio()

    if max(match_name, match_title) >= 0.55:
        return True, max(match_name, match_title)

    else:
        return False, (match_name + match_title) / 2


def get_lyrics_letssingit(song_name):
    """
    Scrapes the lyrics of a song since spotify does not provide lyrics
    takes song title as arguement
    """

    lyrics = ''
    url = "http://search.letssingit.com/cgi-exe/am.cgi?a=search&artist_id=&l=archive&s=" + \
        quote(song_name.encode('utf-8'))
    html = urlopen(url).read()
    soup = BeautifulSoup(html, "html.parser")
    link = soup.find('a', {'class': 'high_profile'})

    try:
        link = link.get('href')
        link = urlopen(link).read()
        soup = BeautifulSoup(link, "html.parser")

        lyrics = soup.find('div', {'id': 'lyrics'}).text[3:]

    except:
        pass

    return lyrics

def get_lyrics_genius(song_title):
    """
    Scrapes the lyrics from Genius.com
    """

    base_url = 'http://api.genius.com'
    headers = {'Authorization': 'Bearer %s' %(GENIUS_KEY)}
    search_url = base_url + '/search'
    data = {'q': song_title}

    response = requests.get(search_url, data=data, headers=headers)
    json = response.json()
    song_api_path = json['response']['hits'][0]['result']['api_path']

    song_url = base_url + song_api_path
    response = requests.get(song_url, headers=headers)
    json = response.json()
    path = json['response']['song']['path']
    page_url = 'http://genius.com' + path

    page = requests.get(page_url)
    soup = BeautifulSoup(page.text, 'html.parser')
    div = soup.find('div',{'class': 'song_body-lyrics'})
    lyrics = div.find('p').getText()

    return lyrics


def get_details_spotify(song_name):
    """
    Tries finding metadata through Spotify
    """

    song_name = improvename.songname(song_name)

    spotify = spotipy.Spotify()
    results = spotify.search(song_name, limit=1)  # Find top result

    log.log_indented('* Finding metadata from Spotify.')

    try:
        # Parse json dictionary
        item = results['tracks']['items'][0]
        album = item['album']['name']
        artist = item['album']['artists'][0]['name']
        song_title = results['tracks']['items'][0]['name']

    except IndexError:
        log.log_error(
            '* Could not find metadata from Spotify, trying something else.',
            indented=True)
        return None

    try:
        log.log_indented("* Finding lyrics from Genius.com")
        lyrics = get_lyrics_genius(song_title)
    except:
        log.log_error("* Could not find lyrics from Genius.com, trying something else")
        lyrics = get_lyrics_letssingit(song_title)

    match_bool, score = matching_details(song_name, song_title, artist)
    if match_bool:
        return artist, album, song_title, lyrics, match_bool, score
    else:
        return None


def get_details_letssingit(song_name):
    """
    Gets the song details if song details not found through spotify
    """

    song_name = improvename.songname(song_name)

    letssingit_search_url = 'http://search.letssingit.com/cgi-exe/am.cgi' \
                            '?a=search&artist_id=&l=archive&s='
    url = "{}{}".format(letssingit_search_url, quote(song_name.encode('utf-8')))
    html = urlopen(url).read()
    soup = BeautifulSoup(html, 'html.parser')
    link = soup.find('a', {'class': 'high_profile'})
    try:
        link = link.get('href')
        link = urlopen(link).read()

        soup = BeautifulSoup(link, 'html.parser')

        album_div = soup.find('div', {'id': 'albums'})
        title_div = soup.find('div', {'id': 'content_artist'}).find('h1')

        try:
            lyrics = soup.find('div', {'id': 'lyrics'}).text
            lyrics = lyrics[3:]
        except AttributeError:
            lyrics = ''
            log.log_error('* Couldn\'t find lyrics', indented=True)

        try:
            song_title = title_div.contents[0]
            song_title = song_title[1:-8]
        except AttributeError:
            log.log_error('* Couldn\'t reset song title', indented=True)
            song_title = song_name

        try:
            artist = title_div.contents[1].getText()
        except AttributeError:
            log.log_error('* Couldn\'t find artist name', indented=True)
            artist = 'Unknown'

        try:
            album = album_div.find('a').contents[0]
            album = album[:-7]
        except AttributeError:
            log.log_error('* Couldn\'t find the album name', indented=True)
            album = artist

    except AttributeError:
        log.log_error('* Couldn\'t find song details', indented=True)

        album = song_name
        song_title = song_name
        artist = 'Unknown'
        lyrics = ''

    match_bool, score = matching_details(song_name, song_title, artist)

    return artist, album, song_title, lyrics, match_bool, score


def add_albumart(albumart, song_title):
    """
    Adds the album art to the song
    """

    try:
        img = urlopen(albumart)  # Gets album art from url

    except Exception:
        log.log_error('* Could not add album art', indented=True)
        return None

    audio = EasyMP3(song_title, ID3=ID3)
    try:
        audio.add_tags()
    except _util.error:
        pass

    audio.tags.add(
        APIC(
            encoding=3,  # UTF-8
            mime='image/png',
            type=3,  # 3 is for album art
            desc='Cover',
            data=img.read()  # Reads and adds album art
        )
    )
    audio.save()
    log.log('> Added album art')


def add_details(file_name, title, artist, album, lyrics=''):
    """
    Adds the details to song
    """

    tags = EasyMP3(file_name)
    tags['title'] = title
    tags['artist'] = artist
    tags['album'] = album
    tags.save()

    tags = ID3(file_name)
    uslt_output = USLT(encoding=3, lang=u'eng', desc=u'desc', text=lyrics)
    tags['USLT::\'eng\''] = uslt_output

    tags.save(file_name)

    log.log('> Adding properties')
    log.log_indented('[*] Title: %s' % title)
    log.log_indented('[*] Artist: %s' % artist)
    log.log_indented('[*] Album: %s ' % album)


def fix_music(file_name):
    """
    Searches for '.mp3' files in directory (optionally recursive)
    and checks whether they already contain album art and album name tags or not.
    """

    setup()

    if not six.PY3:
        file_name = file_name.encode('utf-8')

    tags = File(file_name)

    log.log(file_name)
    log.log('> Adding metadata')

    try:
        artist, album, song_name, lyrics, match_bool, score = get_details_spotify(
            file_name)  # Try finding details through spotify

    except Exception:
        artist, album, song_name, lyrics, match_bool, score = get_details_letssingit(
            file_name)  # Use bad scraping method as last resort


    albumart = albumsearch.img_search("{} {}".format(artist, album))

    if match_bool:
        add_albumart(albumart, file_name)
        add_details(file_name, song_name, artist, album, lyrics)

        try:
            rename(file_name, "{0} - {1}.mp3".format(artist, song_name))
        except OSError:
            log.log_error('Couldn\'t rename file')
            pass
    else:
        log.log_error('* Couldn\'t find appropriate details of your song',
                      indented=True)

    log.log("Match score: {}/10.0".format(round(score * 10, 1)))
    log.log_success()


if __name__ == '__main__':
    main()



#!/usr/bin/env python

r"""
  __  __           _      _   _               
 |  \/  |_   _ ___(_) ___| \ | | _____      __
 | |\/| | | | / __| |/ __|  \| |/ _ \ \ /\ / /
 | |  | | |_| \__ \ | (__| |\  | (_) \ V  V / 
 |_|  |_|\__,_|___/_|\___|_| \_|\___/ \_/\_/  
                                              
"""

import argparse
import requests
import six
import spotipy
import youtube_dl

from os import name, system
from os.path import exists
from collections import OrderedDict

if six.PY2:
    import BeautifulSoup
    input = raw_input
elif six.PY3:
    from bs4 import BeautifulSoup

# Project specific inputs
import repair
import log
from utils import *

YOUTUBECLASS = 'spf-prefetch'
clear = 'clear' if name != 'nt' else 'cls'


def add_config():
    """
    Prompts user for API keys, adds them in an .ini file stored in the same
    location as that of the script
    """

    genius_key = input('Enter Genius key : ')
    bing_key = input('Enter Bing key : ')

    CONFIG['keys']['bing_key'] = bing_key
    CONFIG['keys']['genius_key'] = genius_key

    with open(config_path, 'w') as configfile:
        CONFIG.write(configfile)


def get_tracks_from_album(album_name):
    """
    Gets tracks from an album using Spotify's API
    """

    spotify = spotipy.Spotify()

    result = spotify.search(q='album:' + album_name, limit=1)
    result = result['tracks']['items'][0]['album']
    album, album_id, artist = result['name'], result['id'], result['artists'][0]['name']
    result = spotify.album_tracks(album_id=album_id)
    songs = [item['name'] for item in result['items']]
    return songs, album, artist


def get_url(song_input, auto):
    """
    Provides user with a list of songs to choose from
    returns the url of chosen song.
    """

    youtube_list = OrderedDict()
    num = 0  # List of songs index

    html = requests.get('https://www.youtube.com/results',
                        params={'search_query': song_input})
    soup = BeautifulSoup(html.text, 'html.parser')

    # In all Youtube Search Results
    for i in soup.findAll('a', {'rel': YOUTUBECLASS}):
        song_url = 'https://www.youtube.com' + (i.get('href'))
        song_title = i.get('title')
        # Add title and song url to dictionary
        youtube_list.update({song_title: song_url})

        if auto:
            print(song_title)
            return list(youtube_list.values())[0], list(youtube_list.keys())[0]

        num += 1
        print("({0}) {1}".format(num, song_title))  # Prints list

    if not youtube_list:
        log.log_error('No match found on YouTube, try refining your search!')
        exit()

    # Gets and returns the demanded song url and title and url
    return prompt(youtube_list)


def prompt(youtube_list):
    """
    Prompts for song number from list of songs
    """

    option = 0
    while not option:
        try:
            option = int(input('\nEnter song number > '))
        except ValueError:
            print('Value entered not a number. Try again.')

    try:
        song_url = list(youtube_list.values())[option - 1]
        song_title = list(youtube_list.keys())[option - 1]
    except IndexError:
        log.log_error('Invalid Input')
        exit()

    system(clear)
    print('Download Song: ')
    print(song_title)
    print('Y/N?')
    confirm = input('> ')
    if not confirm or confirm.lower() == 'y':
        pass
    elif confirm.lower() == 'n':
        exit()
    else:
        log.log_error('Invalid Input')
        exit()

    return song_url, song_title


def download_song(song_url, song_title):
    """
    Downloads song from youtube-dl
    """
    outtmpl = song_title + '.%(ext)s'
    ydl_opts = {
        'format': 'bestaudio/best',
        'outtmpl': outtmpl,
        'postprocessors': [{
            'key': 'FFmpegExtractAudio',
            'preferredcodec': 'mp3',
            'preferredquality': '192',
        },
            {'key': 'FFmpegMetadata'},
        ],

    }

    with youtube_dl.YoutubeDL(ydl_opts) as ydl:
        info_dict = ydl.extract_info(song_url, download=True)


def music_now(query, auto):
    """
    Finds the YouTube URL of the queried song, downloads
    it as MP3 file and adds the appropriate tags.
    """

    song_url, file_name = get_url(query, auto)
    download_song(song_url, file_name)
    system(clear)
    repair.fix_music(file_name + '.mp3')


def main():
    """
    Starts here, handles arguments
    """

    global clear

    parser = argparse.ArgumentParser(
        description='Download songs with album art and metadata!')
    parser.add_argument('-c', '--config', action='store_true',
                        help='Set your API keys')
    parser.add_argument('-m', '--multiple', action='store', dest='multiple_file',
                        help='Download multiple songs from a text file list. \
                        Tip: write song and artist in the same line')
    parser.add_argument('-a', '--auto', action='store_true',
                        help='Automatically chooses top result')
    parser.add_argument('--album', action='store_true',
                        help='Downloads all songs from an album')
    parser.add_argument('-v', '--verbose', action='store_true',
                        help='Activate verbose output (no console clears)')
    args = parser.parse_args()
    arg_multiple = args.multiple_file or None
    arg_auto = args.auto or None
    arg_album = args.album or None
    arg_config = args.config
    clear = '' if args.verbose else clear

    system(clear)

    setup()

    if arg_config:
        add_config()

    elif arg_multiple and arg_album:
        log.log_error('Incompatible options "multiple" and "album!"')

    elif arg_album:
        album_name = input('Enter album information (name and artist): ')
        try:
            tracks, album, artist = get_tracks_from_album(album_name)
            print("\nArtist: {}\nAlbum: {}\nSongs:\n\t{}".format(
                  artist, album, '\n\t'.join(tracks)))
            confirm = input(
                '\nAre these the songs you want to download? (Y/N)\n> ')

        except IndexError:
            log.log_error('Couldn\'t find album')
            exit()

        if not confirm or confirm.lower() == ('y'):
            for track_name in tracks:
                music_now("{} {}".format(track_name, artist), arg_auto)

        elif confirm.lower() == 'n':
            log.log_error('Sorry if appropriate results weren\'t found.' \
                          'Try adding more album information such as artist.')
            exit()
        else:
            log.log_error('Invalid Input')
            exit()

    elif arg_multiple:
        if not exists(arg_multiple):
            log.log_error('Requested file for multiple download "{}" ' \
                          'not found'.format(arg_multiple))
            exit()

        with open(arg_multiple, 'r') as f:
            file_names = [line.strip() for line in f if line.strip()]

        for files in file_names:
            music_now(files, arg_auto)

    else:
        query = input('Enter Song Information: ')
        music_now(query, arg_auto)


if __name__ == '__main__':
    main()




#!/usr/bin/env python

import six
from os.path import basename, realpath

if six.PY2:
    import ConfigParser as configparser
elif six.PY3:
    import configparser

CONFIG = ''
BING_KEY = ''
GENIUS_KEY = ''
config_path = ''
LOG_FILENAME = 'musicrepair_log.txt'
LOG_LINE_SEPARATOR = '........................\n'

def setup():
    """
    Gathers all configs
    """

    global CONFIG, BING_KEY, GENIUS_KEY, config_path, LOG_FILENAME, LOG_LINE_SEPARATOR


    CONFIG = configparser.ConfigParser()
    config_path = realpath(__file__).replace(basename(__file__), 'config.ini')
    CONFIG.read(config_path)

    GENIUS_KEY = CONFIG['keys']['genius_key']
    BING_KEY = CONFIG['keys']['bing_key']

    if GENIUS_KEY == '<insert genius key here>':
        log.log_warn('Warning, you are missing the Genius key. Add it using --config')

    if BING_KEY == '<insert bing key here>':
        log.log_warn('Warning, you are missing the Bing key. Add it using --config')




"""
Return Album Art url
"""

import json
import requests
import six
import spotipy

from os.path import realpath, basename

if six.PY2:
    import BeautifulSoup
    from urllib2 import quote, Request, urlopen
elif six.PY3:
    from bs4 import BeautifulSoup
    from urllib.parse import quote
    from urllib.request import Request, urlopen

# Project specific inputs
import log
from utils import *

def img_bing(album):
    """
    Bing image search
    """

    setup()

    log.log_indented('* Trying to extract album art from Bing')

    album += " Album Art"

    api_key = "Key"
    endpoint = "https://api.cognitive.microsoft.com/bing/v5.0/images/search"
    links_dict = {}

    headers = {'Ocp-Apim-Subscription-Key': str(BING_KEY)}
    param = {'q': album, 'count': '1'}

    response = requests.get(endpoint, headers=headers, params=param)
    response = response.json()

    key = 0
    try:
        for i in response['value']:
            links_dict[str(key)] = str((i['contentUrl']))
            key = key + 1

        return links_dict["0"]

    except KeyError:
        return None

def img_google(album):
    """
    Google image search
    """

    log.log_indented('* Trying to extract album art from Google')

    album += " Album Art"
    url = ("https://www.google.com/search?q=" +
           quote(album.encode('utf-8')) + "&source=lnms&tbm=isch")
    header = {'User-Agent':
              '''Mozilla/5.0 (Windows NT 6.1; WOW64)
              AppleWebKit/537.36 (KHTML,like Gecko)
              Chrome/43.0.2357.134 Safari/537.36'''
             }

    soup = BeautifulSoup(urlopen(Request(url, headers=header)), "html.parser")

    albumart_div = soup.find("div", {"class": "rg_meta"})
    albumart = json.loads(albumart_div.text)["ou"]

    return albumart

def img_spotify(query):
    log.log_indented('* Trying to extract album art from Spotify')
    spotify = spotipy.Spotify()

    album = spotify.search(q='album:' + query, limit=1)
    return album['tracks']['items'][0]['album']['images'][0]['url']

def img_search(query):
    return img_spotify(query) or img_google(query) or img_bing(query)

